from plone import api
from plone.api.portal import get_tool
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.dexterity.utils import safe_utf8
from urlparse import urlparse
from zope.component import getUtility


class BrokenLink(object):
    table_attrs = [
        'is_internal',
        'link_origin',
        'link_target',
        'status_code',
        'content_type',
        'response_time',
        'error_message',
        'creator'
    ]

    def __init__(self):
        self.is_broken = None
        self.is_internal = None
        self.link_origin = 'Unknown origin'
        self.link_target = 'Unknown target'
        self.status_code = 'Unknown status code'
        self.content_type = 'Unknown content type'
        self.response_time = 'Unknown response time'
        self.error_message = 'No error occurred'
        self.creator = 'Unknown creator'

    def __iter__(self):
        for attr in self.table_attrs:
            value = getattr(self, attr, '')
            yield value if isinstance(value, basestring) else ''

    def complete_information_with_internal_path(self, obj_having_path, path):
        # relation not broken if possible to traverse to
        try:
            path = urlparse(
                safe_utf8(path).rstrip('/view').rstrip('/download')).path
            storage = getUtility(IRedirectionStorage)
            path = storage.get(path, path)
            api.portal.get().unrestrictedTraverse(path)
            self.is_broken = False
            self.is_internal = True
        except Exception:
            self.is_broken = True
            self.is_internal = True
            self.link_origin = '/'.join(obj_having_path.getPhysicalPath())
            self.link_target = path
            self.creator = obj_having_path.Creator()

    def complete_information_with_external_path(self, obj_having_path, url):
        self.is_internal = False
        self.link_origin = '/'.join(obj_having_path.getPhysicalPath())
        self.link_target = url
        self.creator = obj_having_path.Creator()

    def complete_information_with_internal_uid(self, obj_having_uid, uid):
        portal_catalog = get_tool('portal_catalog')
        if not list(portal_catalog.unrestrictedSearchResults(UID=uid)):
            self.is_broken = True
            self.is_internal = True
            self.link_origin = '/'.join(obj_having_uid.getPhysicalPath())
            self.link_target = uid
            self.creator = obj_having_uid.Creator()
        else:
            self.is_broken = False
            self.is_internal = True

    def complete_information_for_broken_relation_with_broken_relation_obj(
            self,
            obj_having_broken_relation, field):
        self.is_broken = True
        self.is_internal = True
        self.link_origin = '/'.join(
            obj_having_broken_relation.getPhysicalPath())
        self.link_target = 'Broken link in field: ' + str(field)
        self.creator = obj_having_broken_relation.Creator()



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from plone.app.textfield import RichTextValue
from z3c.relationfield import RelationValue
from zope.component.hooks import setSite


def add_textarea_to_page(portal):
    setSite(portal)
    content_page = create(Builder('sl content page').titled(u'contentpage'))
    textarea_having_link = create(Builder('sl textblock')
                                  .within(content_page)
                                  .having(text=RichTextValue('...'))
                                  .titled('textarea'))
    return textarea_having_link


def add_archetype_link_to_plone_site(portal):
    fti = portal.portal_types.get('ftw.simplelayout.ContentPage')
    fti.allowed_content_types = tuple(
        list(fti.allowed_content_types) + ['Link'])
    page = create(Builder('sl content page'))
    link_to_delete = create(Builder('link').titled(u'test link')
                            .within(page)
                            .having(remoteUrl=portal.absolute_url()))
    link_which_stays = create(Builder('link').titled(u'test link')
                              .within(page)
                              .having(remoteUrl=portal.absolute_url()))

    create(Builder('link').titled(u'archetype link')
           .within(page)
           .having(remoteUrl=portal.absolute_url()))
    create(Builder('link').titled(u'archetype link').within(page).having(
        remoteUrl=portal.absolute_url() + '/ImWearingAnInvisibilityCloak'))
    create(Builder('link').titled(u'archetype link')
           .within(page)
           .having(remoteUrl=portal.absolute_url(),
                   relatedItems=link_which_stays.UID()))
    create(Builder('link').titled(u'archetype link')
           .within(page)
           .having(remoteUrl=portal.absolute_url(),
                   relatedItems=link_to_delete.UID()))

    # delete this obj, so the reference will be broken
    parent = link_to_delete.aq_parent
    parent.manage_delObjects([link_to_delete.getId()])


def set_up_test_environment(portal):
    """Set up two plone sites. Both plone sites having the same content.
    Plone sites have working and broken relations and external links.
    """

    setSite(portal)
    pages = [create(
        Builder('sl content page').titled(u'Page {}'.format(index)))
        for index in range(9)]

    # create broken relation value
    broken_rel = RelationValue(pages[2])
    broken_rel.to_id = None

    content_for_the_setup = [
        {
            'test_number': '1',
            'content_type': 'sl textblock',
            'within': pages[0],
            'title': '1',
            'int_url': None,
            'ext_url': portal.absolute_url(),
            'text': None,
            'textblock_within1': None,
            'textblock_url1': None,
            'textblock_within2': None,
            'textblock_url2': None,
        },
        {
            'test_number': '1',
            'content_type': 'sl textblock',
            'within': pages[0],
            'title': '0',
            'int_url': None,
            'ext_url': portal.absolute_url() + '/gibtsnicht',
            'text': None,
            'textblock_within1': None,
            'textblock_url1': None,
            'textblock_within2': None,
            'textblock_url2': None,
        },
        {
            'test_number': '2',
            'content_type': 'sl textblock',
            'within': pages[0],
            'title': 'default title',
            'int_url': RelationValue(pages[1]),
            'ext_url': None,
            'text': None,
            'textblock_within1': None,
            'textblock_url1': None,
            'textblock_within2': None,
            'textblock_url2': None,
        },
        {
            'test_number': '2',
            'content_type': 'sl textblock',
            'within': pages[0],
            'title': 'broken relation',
            'int_url': broken_rel,
            'ext_url': None,
            'text': None,
            'textblock_within1': None,
            'textblock_url1': None,
            'textblock_within2': None,
            'textblock_url2': None,
        },
        {
            'test_number': '3',
            'content_type': 'sl content page',
            'within': pages[3],
            'title': 'Content page on page 3',
            'int_url': None,
            'ext_url': None,
            'text': None,
            'textblock_within1': pages[3],
            'textblock_url1': 'content-page-on-page-3',
            'textblock_within2': pages[3],
            'textblock_url2': 'Idunnoexist',
        },
        {
            'test_number': '4',
            'content_type': 'sl content page',
            'within': pages[4],
            'title': 'Content page on page 4',
            'int_url': None,
            'ext_url': None,
            'text': None,
            'textblock_within1': pages[4],
            'textblock_url1': './content-page-on-page-4',
            'textblock_within2': pages[4],
            'textblock_url2': './Icantbefound',
        },
        {
            'test_number': '5',
            'content_type': 'sl content page',
            'within': pages[5],
            'title': 'Content page on page 5',
            'int_url': None,
            'ext_url': None,
            'text': None,
            'textblock_within1': pages[5],
            'textblock_url1': '/page-5',
            'textblock_within2': pages[5],
            'textblock_url2': '/Iwasnevercreated',
        },
        {
            'test_number': '6',
            'content_type': 'sl content page',
            'within': pages[6],
            'title': 'Content page on page 6',
            'int_url': None,
            'ext_url': None,
            'text': None,
            'textblock_within1': pages[6],
            'textblock_url1': portal.absolute_url(),
            'textblock_within2': pages[6],
            'textblock_url2': portal.absolute_url() + '/Sadnottoexist',
        },
    ]

    for content in content_for_the_setup:
        make_content_for_me(content['content_type'],
                            content['within'],
                            content['title'],
                            content['int_url'],
                            content['ext_url'],
                            content['text'],
                            content['textblock_within1'],
                            content['textblock_url1'],
                            content['textblock_within2'],
                            content['textblock_url2'], )


def make_content_for_me(content_type, within, title, int_url, ext_url, text,
                        textblock_within1, textblock_url1, textblock_within2,
                        textblock_url2):
    create(Builder(content_type)
           .within(within)
           .titled(title.decode('utf-8'))
           .having(internal_link=int_url,
                   external_link=ext_url,
                   text=text))

    if textblock_within1 and textblock_url1:
        add_link_into_textarea_without_using_the_browser(textblock_within1,
                                                         textblock_url1)

    if textblock_within2 and textblock_url2:
        add_link_into_textarea_without_using_the_browser(textblock_within2,
                                                         textblock_url2)


def add_link_into_textarea_without_using_the_browser(page, url):
    create(Builder('sl textblock')
           .within(page)
           .having(text=RichTextValue('<a href="%s">a link</a>' % url))
           .titled('A textblock link not using the browser'))



from plone.testing.z2 import ZSERVER_FIXTURE
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import FunctionalTesting
from ftw.linkchecker.tests import builders
from plone.app.testing import PLONE_SITE_ID
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import pushGlobalRegistry
from plone.app.testing.helpers import login, logout
from plone.app.testing.layers import PloneFixture
from plone.testing import z2
from plone.testing import zca, security
from plone.testing import zodb
from zope.configuration import xmlconfig
import contextlib
from plone.app.testing.interfaces import (
    DEFAULT_LANGUAGE,
    TEST_USER_ID,
    TEST_USER_NAME,
    TEST_USER_PASSWORD,
    TEST_USER_ROLES,
    SITE_OWNER_NAME,
)

ADDITIONAL_PAGES_TO_SETUP = [
    {
        'page_id': 'plone2',
        'page_title': u'Plone site two',
    },
]


class PloneFixtureChild(PloneFixture):
    def __init__(self):
        super(PloneFixture, self).__init__()

    # overwrite setUp() to also call self.setUpDefaultContent_AdditionalPage()
    def setUp(self):
        self['zodbDB'] = zodb.stackDemoStorage(self.get('zodbDB'),
                                               name='PloneFixture')
        self.setUpZCML()
        with z2.zopeApp() as app:
            self.setUpProducts(app)
            self.setUpDefaultContent(app)
            for ADDITIONAL_PAGE in ADDITIONAL_PAGES_TO_SETUP:
                self.setUpDefaultContent_AdditionalPage(app, ADDITIONAL_PAGE[
                    'page_id'], ADDITIONAL_PAGE['page_title'])

    def setUpDefaultContent_AdditionalPage(self, app, additional_page_id,
                                           additional_page_title):
        z2.login(app['acl_users'], SITE_OWNER_NAME)
        from Products.CMFPlone.factory import addPloneSite
        # Set up the second page with another ID here.
        addPloneSite(app, additional_page_id,
                     title=additional_page_title,
                     setup_content=False,
                     default_language=DEFAULT_LANGUAGE,
                     extension_ids=self.extensionProfiles,
                     )
        app[additional_page_id]['portal_workflow'].setDefaultChain('')
        pas = app[additional_page_id]['acl_users']
        pas.source_users.addUser(
            TEST_USER_ID,
            TEST_USER_NAME,
            TEST_USER_PASSWORD)
        for role in TEST_USER_ROLES:
            pas.portal_role_manager.doAssignRoleToPrincipal(TEST_USER_ID,
                                                            role)
        z2.logout()


PLONE_FIXTURE = PloneFixtureChild()


class LinkcheckerLayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER, ZSERVER_FIXTURE)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.linkchecker')
        z2.installProduct(app, 'ftw.simplelayout')

    def setUp(self):
        self['zodbDB'] = zodb.stackDemoStorage(self.get('zodbDB'),
                                               name=self.__name__)
        name = self.__name__ if self.__name__ is not None else 'not-named'
        contextName = "PloneSandboxLayer-%s" % name
        self['configurationContext'] = configurationContext = (
            zca.stackConfigurationContext(self.get('configurationContext'),
                                          name=contextName))
        # call ploneSite() for all pages in dict.
        plone_site_ids = [PLONE_SITE_ID]
        plone_site_ids.extend(additional_page['page_id'] for additional_page in
                              ADDITIONAL_PAGES_TO_SETUP)
        for plone_site_id in plone_site_ids:
            with self.ploneSite(plone_site_id) as portal:
                from zope.site.hooks import setSite, setHooks
                setHooks()
                setSite(None)
                pushGlobalRegistry(portal)
                security.pushCheckers()
                from Products.PluggableAuthService.PluggableAuthService import (
                    MultiPlugins)
                preSetupMultiPlugins = MultiPlugins[:]
                self.setUpZope(portal.getPhysicalRoot(), configurationContext)
                setSite(portal)
                self.setUpPloneSite(portal)
                setSite(None)
            self.snapshotMultiPlugins(preSetupMultiPlugins)

    @contextlib.contextmanager
    def ploneSite(self, plone_site_id, db=None, connection=None, environ=None):
        from zope.site.hooks import setSite, getSite, setHooks
        setHooks()
        site = getSite()
        with z2.zopeApp(db, connection, environ) as app:
            portal = app[plone_site_id]
            setSite(portal)
            login(portal, TEST_USER_NAME)
            try:
                yield portal
            finally:
                logout()
                if site is not portal:
                    setSite(site)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.simplelayout.contenttypes:default')


LINKCHECKER_FIXTURE = LinkcheckerLayer()
LINKCHECKER_FUNCTIONAL = FunctionalTesting(
    bases=(LINKCHECKER_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.linkchecker:functional")



from functools import partial
from multiprocessing import Pool
import os
import requests
import time


def millis():
    return int(round(time.time() * 1000))


def get_uri_response(external_link_obj, timeout):
    headers = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'
    }
    error = None
    response = None
    start_time = millis()
    try:
        response = requests.head(external_link_obj.link_target,
                                 timeout=timeout,
                                 headers=headers,
                                 allow_redirects=False,
                                 verify=False)
    except requests.exceptions.Timeout:
        error = 'Timeout'
    except requests.exceptions.TooManyRedirects:
        error = 'Too many redirects'
    except requests.exceptions.ConnectionError:
        error = 'Connection Error'
    except Exception as e:
        error = e.message

    time = millis() - start_time

    if response and response.status_code == 200 \
            or 'resolveuid' in external_link_obj.link_target:
        external_link_obj.is_broken = False
    else:
        external_link_obj.is_broken = True
        external_link_obj.status_code = getattr(response, 'status_code', None)
        external_link_obj.content_type = headers.get('Content-Type', None)
        external_link_obj.response_time = time
        external_link_obj.error_message = error

    return external_link_obj


def limit_cpu():
    os.nice(19)


def work_through_urls(external_link_objs, timeout_config):
    part_get_uri_response = partial(get_uri_response, timeout=timeout_config)
    pool = Pool(initializer=limit_cpu)
    start_time = millis()
    external_link_objs = pool.map(part_get_uri_response, external_link_objs)
    total_time = millis() - start_time
    return external_link_objs, total_time



from Testing.makerequest import makerequest
from email import encoders
from email.header import Header
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate
from zope.component.hooks import setSite
import plone.api
import time


class MailSender(object):
    def __init__(self, portal=None):
        if not portal:
            self.app = globals()['app']
            self.app = makerequest(self.app)
            self.PLONE = self.app.get('Plone')
        if portal:
            self.PLONE = portal
        setSite(self.PLONE)

    def send_feedback(self, email_subject, email_message,
                      receiver_email_address, xlsx_file, file_name):
        """Send an email including an excel workbook attached.
        """
        mh = plone.api.portal.get_tool('MailHost')
        from_name = self.PLONE.getProperty('email_from_name', '')
        from_email = self.PLONE.getProperty('email_from_address', '')

        sender = 'Linkcheck Reporter'
        recipient = from_email

        msg = MIMEMultipart()
        msg['From'] = "%s <%s>" % (from_name, from_email)
        msg['reply-to'] = "%s <%s>" % (sender, recipient)
        msg['To'] = receiver_email_address
        msg['Date'] = formatdate(localtime=True)
        msg['Subject'] = Header(email_subject, 'windows-1252')
        msg.attach(MIMEText(email_message.encode(
            'windows-1252'), 'plain', 'windows-1252'))
        part = MIMEBase('application', "octet-stream")
        part.set_payload(xlsx_file.read())
        encoders.encode_base64(part)
        part.add_header(
            'Content-Disposition',
            'attachment; filename="' + file_name + '"'
        )
        msg.attach(part)

        mh.send(msg, mto=receiver_email_address,
                mfrom=from_email,
                immediate=True)



from ftw.linkchecker import report_generating
from ftw.linkchecker.cell_format import BOLD
from ftw.linkchecker.cell_format import CENTER
from ftw.linkchecker.cell_format import DEFAULT_FONTNAME
from ftw.linkchecker.cell_format import DEFAULT_FONTSIZE
from ftw.linkchecker.command import broken_link


def generate_test_data_excel_workbook():
    example_data = broken_link.BrokenLink()
    example_data.is_internal = 'Some'
    example_data.link_origin = 'example'
    example_data.link_target = 'data'
    example_data.status_code = 'to'
    example_data.content_type = 'fill'
    example_data.response_time = 'the'
    example_data.error_message = 'sheet'
    example_data.creator = 'slowly'
    exemplar_report_data = [example_data] * 9

    base_uri = 'http://www.example_uri.com'
    file_i = report_generating.ReportCreator()
    file_i.append_report_data(report_generating.LABELS,
                              base_uri,
                              BOLD &
                              CENTER &
                              DEFAULT_FONTNAME &
                              DEFAULT_FONTSIZE)
    file_i.append_report_data(exemplar_report_data,
                              base_uri,
                              DEFAULT_FONTNAME &
                              DEFAULT_FONTSIZE)
    file_i.add_general_autofilter()
    file_i.cell_width_autofitter()
    file_i.safe_workbook()
    xlsx_file = file_i.get_workbook()

    return xlsx_file



from Products.CMFPlone.utils import safe_unicode
from ftw.linkchecker.command import broken_link
from io import BytesIO
from plone import api
import re
import xlsxwriter

LABELS = broken_link.BrokenLink()
LABELS.is_internal = 'Internal/External'
LABELS.link_origin = 'Origin'
LABELS.link_target = 'Destination'
LABELS.status_code = 'Status Code'
LABELS.content_type = 'Content Type'
LABELS.response_time = 'Response Time'
LABELS.error_message = 'Error Message'
LABELS.creator = 'Creator'
LABELS = [LABELS]


class ReportCreator(object):

    def __init__(self):
        self.row = 0
        self.output_xlsx = BytesIO()
        self.workbook = xlsxwriter.Workbook(self.output_xlsx,
                                            {'in_memory': True,
                                             'strings_to_url': False})
        self.worksheet = self.workbook.add_worksheet()
        self.table = []

    def append_report_data(self, link_objs, base_uri, format=None):
        format = format.get_workbook_format(self.workbook) if format else None
        for link_obj in link_objs:

            int_ext_link = link_obj.is_internal
            if link_obj.is_internal is None:
                int_ext_link = 'Unknown'
            elif link_obj.is_internal is True:
                int_ext_link = 'Internal Link'
                link_obj.status_code = 404
            elif link_obj.is_internal is False:
                int_ext_link = 'External Link'

            # remove portal path in link_target and link_origin
            portal_path_segments = api.portal.get().getPhysicalPath()
            portal_reg = re.compile('^' + '/'.join(portal_path_segments))
            link_obj.link_origin = re.sub(portal_reg, safe_unicode(base_uri),
                                          safe_unicode(link_obj.link_origin))
            link_obj.link_target = re.sub(portal_reg, safe_unicode(base_uri),
                                          safe_unicode(link_obj.link_target))

            self.worksheet.write(self.row, 0, int_ext_link, format)
            self.worksheet.write_string(self.row, 1,
                                        safe_unicode(link_obj.link_origin),
                                        format)
            self.worksheet.write_string(self.row, 2,
                                        safe_unicode(link_obj.link_target),
                                        format)
            self.worksheet.write(self.row, 3,
                                 safe_unicode(link_obj.status_code), format)
            self.worksheet.write(self.row, 4,
                                 safe_unicode(link_obj.content_type), format)
            self.worksheet.write(self.row, 5,
                                 safe_unicode(link_obj.response_time), format)
            self.worksheet.write(self.row, 6,
                                 safe_unicode(link_obj.error_message), format)
            self.worksheet.write(self.row, 7,
                                 safe_unicode(link_obj.creator), format)

            self.row += 1
        self.table.extend(link_objs)

    def add_general_autofilter(self):
        self.worksheet.autofilter(0, 0, self.row,
                                  len(LABELS[0].table_attrs) - 1)

    def get_column_widths(self):
        columns_size = [0] * len(self.table[0].table_attrs)
        for row in self.table:
            for j, column_element in enumerate(row):
                columns_size[j] = max(columns_size[j], len(column_element))
                # enlarge column width by content up to 100 characters
                if columns_size[j] > 100:
                    columns_size[j] = 100
        return columns_size

    def cell_width_autofitter(self):
        column_widths = self.get_column_widths()
        for row, column_width in enumerate(column_widths):
            self.worksheet.set_column(self.row, row, column_width + 7)

    def safe_workbook(self):
        self.workbook.close()

    def get_workbook(self):
        self.output_xlsx.seek(0)
        return self.output_xlsx



from ftw.linkchecker import report_generating
from ftw.linkchecker import report_mailer
from ftw.linkchecker.cell_format import BOLD
from ftw.linkchecker.cell_format import CENTER
from ftw.linkchecker.cell_format import DEFAULT_FONTNAME
from ftw.linkchecker.cell_format import DEFAULT_FONTSIZE
from ftw.linkchecker.command.checking_links import get_file_name
from ftw.linkchecker.command import broken_link
from ftw.linkchecker.command import checking_links
from ftw.linkchecker.tests import ArchetypeFunctionalTestCase
from ftw.linkchecker.tests import FunctionalTestCase
from ftw.linkchecker.tests import MultiPageTestCase
from ftw.linkchecker.tests.exemplar_data.exemplar_config import config_file
from ftw.linkchecker.tests.helper_generating_excel import generate_test_data_excel_workbook
from ftw.testing.mailing import Mailing
from plone import api
from zope.component.hooks import setSite
import email
import os
import pandas as pd

CURRENT_PATH = os.path.dirname(os.path.abspath(__file__))


class TestArchetypeLink(ArchetypeFunctionalTestCase):

    def test_external_link(self):
        self.helper_check_links()

        self.assertIn('/plone/ftw-simplelayout-contentpage/archetype-link-1',
                      self.paths_from,
                      'Testing a broken external archetype link: It is'
                      'expected that we find archetype-link-1 in the broken'
                      'link list because it is pointing to '
                      '"http://localhost:55001/plone/ImWearingAnInvisibilityCloak".')

        self.assertNotIn('/plone/ftw-simplelayout-contentpage/archetype-link',
                         self.paths_from,
                         'Testing a working external archetype link: It is'
                         'expected that we don\'t find archetype-link in the '
                         'broken link list because it is pointing to '
                         '"http://localhost:55001/plone".')

    def test_relation(self):
        self.helper_check_links()

        self.assertIn('/plone/ftw-simplelayout-contentpage/archetype-link-3',
                      self.paths_from,
                      'Testing a broken archetype relation: It is expected '
                      'that we find archetype-link-3 in the broken link list '
                      'because it is pointing to a deleted object.')

        self.assertNotIn(
            '/plone/ftw-simplelayout-contentpage/archetype-link-2',
            self.paths_from,
            'Testing a valid archetype relation: It is expected '
            'that we don\'t find archetype-link-2 in the broken '
            'link list because it is pointing to a valid object.')

    def helper_check_links(self):
        setSite(self.portal)
        checking_links.setup_plone(self.app, self.plone_site_objs[0])
        email_address, base_uri, timeout_config, upload_location = checking_links.extract_config(
            config_file)
        broken_relations_and_links_info = checking_links.get_total_fetching_time_and_broken_link_objs(
            int(timeout_config))
        self.paths_from = [list_element.link_origin for list_element in
                           broken_relations_and_links_info[1]]


class TestFindingLinksAndRelations(MultiPageTestCase):

    def test_different_emails_for_different_plone_sites(self):
        self.assertIn(
            self.portal,
            self.plone_site_objs,
            'It is expected that the first plone site is in the list of'
            'plone sites found.')

        self.assertIn(
            self.portals[1],
            self.plone_site_objs,
            'It is expected that the second plone site is in the list of'
            'plone sites found.')

    def test_email_addresses_correspond_to_correct_plone_site(self):
        setSite(self.plone_site_objs[0])
        email_address_0, base_uri0, timeout_config0, upload_location0 = checking_links.extract_config(config_file)
        setSite(self.plone_site_objs[1])
        email_address_1, base_uri1, timeout_config1, upload_location1 = checking_links.extract_config(config_file)

        self.assertEqual(
            email_address_0,
            'hugo.boss@4teamwork.ch',
            'It is expected that the email address for page 0 is corresponding'
            'to its test site administrators email (hugo.boss@4teamwork.ch).')

        self.assertEqual(
            email_address_1,
            'berta.huber@gmail.com',
            'It is expected that the email address for page 1 is corresponding'
            'to its test site administrators email (berta.huber@gmail.com).')

        self.assertEqual(base_uri0, 'http://example1.ch',
                         'It is expected that the value was extracted from '
                         'the dictionary.')

        self.assertEqual(timeout_config0, '1',
                         'It is expected that the value was extracted from '
                         'the dictionary.')

    def helper_function_getting_getting_link_information(self):
        checking_links.setup_plone(self.app, self.plone_site_objs[0])
        email_address, base_uri, timeout_config, upload_location = checking_links.extract_config(
            config_file)
        broken_relations_and_links_info = checking_links.get_total_fetching_time_and_broken_link_objs(
            int(timeout_config))
        self.paths_from = [list_element.link_origin for list_element in
                           broken_relations_and_links_info[1]]

    def test_external_link(self):
        # Test 1 - External link in link field
        self.helper_function_getting_getting_link_information()
        self.assertIn(
            '/plone/page-0/0', self.paths_from,
            'Testing an invalid external link in IURI: We expect finding '
            '"/plone/page-0/0" in broken_relations_and_links_info because it '
            'is linking to an invalid link'
            '(http://localhost/plone/gibtsnicht).'
        )
        self.assertNotIn(
            '/plone/page-0/1', self.paths_from,
            'Testing valid external link in IURI: We expect not to find '
            '"/plone/page-0/1" in broken_relations_and_links_info'
            'because it links to a valid site'
            '(http://localhost/plone).'
        )

    def test_relation_values(self):
        # Test 2 - Relation in relation field
        self.helper_function_getting_getting_link_information()
        self.assertIn(
            '/plone/page-0/broken-relation', self.paths_from,
            'Testing an invalid relation in IRealtion: We expect finding '
            '"/plone/page-0/broken-relation" in broken_relations_and_links_info'
            'because it links a deleted plone site'
            '(http://localhost/plone/page-2).'
        )
        self.assertNotIn(
            '/plone/page-0/default-title', self.paths_from,
            'Testing valid relation in IRelation: We expect not to find '
            '"/plone/page-0/default-title" in broken_relations_and_links_info'
            'because it links to a valid site'
            '(http://localhost/plone/page-1).'
        )

    def test_relations_in_textarea_type1(self):
        # Test 3 - Relation in textarea (link like -> foo)
        self.helper_function_getting_getting_link_information()
        self.assertIn(
            '/plone/page-3/a-textblock-link-not-using-the-browser-1',
            self.paths_from,
            'Testing broken relation in textarea: We expect finding'
            '"/plone/page-3/a-textblock-link-not-using-the-browser-1" in'
            'broken_relations_and_links_info because it links to a broken'
            'relation type 1 (Idunnoexist).'
        )
        self.assertNotIn(
            '/plone/page-3/a-textblock-link-not-using-the-browser',
            self.paths_from,
            'Testing valid relation in textarea: We expect not to find'
            '"/plone/page-3/a-textblock-link-not-using-the-browser" in'
            'broken_relations_and_links_info because it links to a valid'
            'relation type 1 (content-page-on-page-3)'
        )

    def test_relations_in_textarea_type2(self):
        # Test 4 - Relation in textarea (link like -> ./foo)
        self.helper_function_getting_getting_link_information()
        self.assertIn(
            '/plone/page-4/a-textblock-link-not-using-the-browser-1',
            self.paths_from,
            'Testing broken relation in textarea: We expect finding'
            '"/plone/page-4/a-textblock-link-not-using-the-browser-1" in'
            'broken_relations_and_links_info because it links to a broken'
            'relation type 2 (./Icantbefound).'
        )
        self.assertNotIn(
            '/plone/page-4/a-textblock-link-not-using-the-browser',
            self.paths_from,
            'Testing valid relation in textarea: We expect not to find'
            '"/plone/page-4/a-textblock-link-not-using-the-browser" in'
            'broken_relations_and_links_info because it links to a valid'
            'relation type 2 (./content-page-on-page-4).'
        )

    def test_relations_in_textarea_type3(self):
        # Test 5 - Relation in textarea (link like -> /foo)
        self.helper_function_getting_getting_link_information()
        self.assertIn(
            '/plone/page-5/a-textblock-link-not-using-the-browser-1',
            self.paths_from,
            'Testing broken relation in textarea: We expect finding'
            '"/plone/page-5/a-textblock-link-not-using-the-browser-1" in'
            'broken_relations_and_links_info because it links to a broken'
            'relation type 3 (/Iwasnevercreated).'
        )
        self.assertNotIn(
            '/plone/page-5/a-textblock-link-not-using-the-browser',
            self.paths_from,
            'Testing valid relation in textarea: We expect not to find'
            '"/plone/page-5/a-textblock-link-not-using-the-browser" in'
            'broken_relations_and_links_info because it links to a valid'
            'relation type 3 (./content-page-on-page-5).'
        )

    def test_external_link_in_textarea(self):
        # Test 6 - External link in textarea (link like -> http://...)
        self.helper_function_getting_getting_link_information()
        self.assertIn(
            '/plone/page-6/a-textblock-link-not-using-the-browser-1',
            self.paths_from,
            'Testing broken link in textarea: We expect finding'
            '"/plone/page-6/a-textblock-link-not-using-the-browser-1" in'
            'broken_relations_and_links_info because it links to a broken'
            'url (http://localhost/Sadnottoexist).'
        )
        self.assertNotIn(
            '/plone/page-6/a-textblock-link-not-using-the-browser',
            self.paths_from,
            'Testing valid link in textarea: We expect not to find'
            '"/plone/page-6/a-textblock-link-not-using-the-browser" in'
            'broken_relations_and_links_info because it links to a valid'
            'url (http://localhost/plone).'
        )


class TestLinksInTextArea(FunctionalTestCase):

    def test_if_paths_from_textareas_are_correctly_joined_type_1(self):
        obj = self.textarea
        url = 'foo'
        res_path = checking_links.create_path_even_there_are_parent_pointers(
            obj, url)

        self.assertEqual(
            res_path,
            '/plone/contentpage/foo',
            'It is expected, that paths like "foo" are appended to the parent'
            'of the textarea.')

    def test_if_paths_from_textareas_are_correctly_joined_type_2(self):
        obj = self.textarea
        url = './foo'
        res_path = checking_links.create_path_even_there_are_parent_pointers(
            obj, url)

        self.assertEqual(
            res_path,
            '/plone/contentpage/foo',
            'It is expected, that paths like "./foo" are appended to the '
            'parent of the textarea.')

    def test_if_paths_from_textareas_are_correctly_joined_type_3(self):
        obj = self.textarea
        url = '/foo'
        res_path = checking_links.create_path_even_there_are_parent_pointers(
            obj, url)

        self.assertEqual(
            res_path,
            '/plone/foo',
            'It is expected, that paths like "/foo" are appended to the site'
            'root.')

    def test_if_paths_from_textareas_are_correctly_joined_type_4(self):
        obj = self.textarea
        url = '../../../../foo'
        res_path = checking_links.create_path_even_there_are_parent_pointers(
            obj, url)

        self.assertEqual(
            res_path,
            '/plone/foo',
            'It is expected, that paths like "../../../../foo" (which point to'
            ' a parent further up than actually existing) are appended to the '
            'site root.')


class TestShippingInformation(FunctionalTestCase):

    def test_if_excel_generator_adds_content_correctly(self):
        """Test if an excel workbook generated by the linkchecker does not
        differ an exemplar workbook containing the expected data.
        """
        path_of_excel_workbook_exemplar = (
                CURRENT_PATH +
                '/exemplar_data/expected_excel_sheet_outcome.xlsx')
        xlsx_file = generate_test_data_excel_workbook()

        # import the excel workbooks as pandas dataframes
        df1 = pd.read_excel(xlsx_file)
        df2 = pd.read_excel(open(path_of_excel_workbook_exemplar, "rb"))

        diff_xlsx_files = pd.concat([df1, df2]).drop_duplicates(keep=False)

        assert df1.equals(df2), \
            "The examplar excel workbook converges from the one generated." \
            "The diff occured in following lines \n\n{}".format(
                diff_xlsx_files)

    def test_if_mail_sender_sending_mail_incl_attachement(self):
        """Test if the mail sent by linkchecker can be received correctly.
        """

        # test email variables
        email_subject = 'Linkchecker Report'
        email_message = 'Dear Site Administrator, \n\n\
            Please check out the linkchecker report attached to this mail.\n\n\
            Friendly regards,\n\
            your 4teamwork linkcheck reporter'
        receiver_email_address = 'hugo@boss.ch'
        report_path = CURRENT_PATH + \
                      '/exemplar_data/expected_excel_sheet_outcome.xlsx'
        exemplar_xlsx_file = open(report_path, "rb")

        # setUp
        Mailing(self.layer['portal']).set_up()
        portal = self.layer['portal']
        setSite(portal)

        file_name = get_file_name()
        report_mailer_instance = report_mailer.MailSender(portal)
        report_mailer_instance.send_feedback(
            email_subject, email_message, receiver_email_address,
            exemplar_xlsx_file, file_name)
        mail = Mailing(portal).pop()
        mail_obj = email.message_from_string(mail)

        self.assertEqual(
            mail_obj.get('To'), 'hugo@boss.ch',
            'The email is expected to be sent to given reveiver.')

        self.assertEqual(
            mail_obj.get_payload()[1].get_content_type(),
            'application/octet-stream',
            'The emails attachement is expected to be a binary file.')

        # tearDown
        Mailing(self.layer['portal']).tear_down()



from ftw.builder import Builder
from ftw.builder import create
from ftw.linkchecker.command import checking_links
from ftw.linkchecker.command.checking_links import get_file_name, extract_config
from ftw.linkchecker.tests import MultiPageTestCase
from ftw.linkchecker.tests.exemplar_data.exemplar_config import config_file
from ftw.linkchecker.tests.helper_generating_excel import generate_test_data_excel_workbook
from ftw.testbrowser import browsing
from zope.component.hooks import setSite
import transaction


class TestReportUploading(MultiPageTestCase):

    @browsing
    def test_report_generated_is_uploaded_to_file_listing_block_if_config_given(self, browser):
        # plone site 0 has an upload config
        setSite(self.plone_site_objs[0])
        page = create(Builder('sl content page').titled(u'Content Page'))
        file_listing_block = create(Builder('sl listingblock')
                                    .titled('File Listing Block')
                                    .within(page))
        xlsx_file = generate_test_data_excel_workbook()
        file_name = get_file_name()
        _, _, _, upload_location = extract_config(config_file)
        checking_links.upload_report_to_filelistingblock(upload_location, xlsx_file, file_name)
        transaction.commit()

        browser.visit(file_listing_block)
        node = browser.css('.linkWrapper a').first
        report_donwload_url = node.attrib.get('href', '')

        # Since we assert a link this is always lower case.
        expected_report_download_url_part = 'plone/content-page/file-listing-block/{}/download'.format(file_name).lower()

        self.assertIn(expected_report_download_url_part,
                      report_donwload_url,
                      'We expect that the report download url is built '
                      'containing the filename and context path.')



from AccessControl.SecurityManagement import newSecurityManager
from Products.Archetypes.Field import ComputedField
from Products.Archetypes.Field import ReferenceField
from Products.Archetypes.Field import StringField
from Products.Archetypes.Field import TextField
from Testing.makerequest import makerequest
from ftw.linkchecker import linkchecker
from ftw.linkchecker import report_generating
from ftw.linkchecker import report_mailer
from ftw.linkchecker import setup_logger
from ftw.linkchecker.cell_format import BOLD
from ftw.linkchecker.cell_format import CENTER
from ftw.linkchecker.cell_format import DEFAULT_FONTNAME
from ftw.linkchecker.cell_format import DEFAULT_FONTSIZE
from ftw.linkchecker.command.broken_link import BrokenLink
from plone import api
from plone.app.textfield.interfaces import IRichText
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import getAdditionalSchemata
from urlparse import urljoin
from urlparse import urlparse
from z3c.relationfield.interfaces import IRelation
from zope.component import getUtility
from zope.component import queryUtility
from zope.component.hooks import setSite
from zope.globalrequest import setRequest
from zope.schema import getFieldsInOrder
from zope.schema.interfaces import IURI
import AccessControl
import argparse
import json
import os
import plone
import re
import time
import transaction


def count_parent_pointers(path_segments):
    if not path_segments:
        return 0
    if path_segments[0] != '..':
        return 0
    for index, segment in enumerate(path_segments):
        if segment == '..':
            continue
        else:
            break
    return index + 1


def create_path_even_there_are_parent_pointers(obj, url):
    portal_path_segments = api.portal.get().getPhysicalPath()
    current_path_segments = obj.aq_parent.getPhysicalPath()
    destination_path_segments = filter(len, url.split('/'))
    destination_path = '/'.join(destination_path_segments)
    portal_path = '/'.join(portal_path_segments)

    number_of_parent_pointers = count_parent_pointers(
        destination_path_segments)
    if number_of_parent_pointers >= len(current_path_segments) - len(
            portal_path_segments):
        new_path = portal_path + '/' + destination_path.lstrip('../')
        return new_path
    else:
        if url.startswith('/'):
            output_path = portal_path + url
        else:
            # XXX: < plone5, relative paths are appended to the basepath having a
            # slash at the end. For plone5 support we need to look at this again.
            output_path = urljoin(
                '/'.join(list(obj.aq_parent.getPhysicalPath()) + ['']), url)

        return output_path


def _get_plone_sites(obj):
    for child in obj.objectValues():
        if child.meta_type == 'Plone Site':
            yield child
        elif child.meta_type == 'Folder':
            for item in _get_plone_sites(child):
                yield item


def setup_plone(app, plone_site_obj):
    app = makerequest(app)
    setRequest(app.REQUEST)
    plone_site_obj = app.unrestrictedTraverse(
        '/'.join(plone_site_obj.getPhysicalPath()))
    user = AccessControl.SecurityManagement.SpecialUsers.system
    user = user.__of__(plone_site_obj.acl_users)
    newSecurityManager(plone_site_obj, user)
    setSite(plone_site_obj)


def get_total_fetching_time_and_broken_link_objs(timeout_config):
    portal_catalog = api.portal.get_tool('portal_catalog')
    brains = portal_catalog.unrestrictedSearchResults()
    link_objs = []
    for brain in brains:
        link_objs.extend(find_links_on_brain_fields(brain))

    external_link_objs = []
    internal_link_objs = []
    for link_obj in link_objs:
        if link_obj.is_internal and link_obj.is_broken:
            internal_link_objs.append(link_obj)
        elif not link_obj.is_internal:
            external_link_objs.append(link_obj)

    external_link_objs_and_total_time = linkchecker.work_through_urls(
        external_link_objs, timeout_config)

    external_link_objs, total_time = external_link_objs_and_total_time

    broken_ext_links = filter(lambda link_obj: link_obj.is_broken,
                              external_link_objs)

    internal_link_objs.extend(broken_ext_links)

    return [total_time, internal_link_objs]


def find_links_on_brain_fields(brain):
    obj = brain.getObject()
    link_objs = []

    if not queryUtility(IDexterityFTI, name=obj.portal_type):
        # is not dexterity
        for field in obj.Schema().fields():
            if not isinstance(field, (TextField, ReferenceField, ComputedField, StringField)):
                continue
            content = field.getRaw(obj)
            if isinstance(field, ReferenceField):
                uid = content
                try:
                    uid_from_relation = obj['at_ordered_refs']['relatesTo']
                except Exception:
                    uid_from_relation = []
                uid.extend(uid_from_relation)
                append_to_link_and_relation_information_for_different_link_types(
                    [[], uid, []], link_objs, obj)

            if not isinstance(content, basestring):
                continue
            # if there is a string having a valid scheme it will be embedded
            # into a href, so we can use the same method as for the dexterity
            # strings and do not need to change the main use case.
            scheme = urlparse(content).scheme
            if scheme and scheme in ['http', 'https']:
                content = 'href="%s"' % content
            extract_and_append_link_objs(content, obj, link_objs)

    if queryUtility(IDexterityFTI, name=obj.portal_type):
        for name, field, schemata in iter_fields(obj.portal_type):
            storage = schemata(obj)
            fieldvalue = getattr(storage, name)
            if not fieldvalue:
                continue

            if IRelation.providedBy(field):
                if fieldvalue.isBroken():
                    link = BrokenLink()
                    link.complete_information_for_broken_relation_with_broken_relation_obj(
                        obj, field)
                    link_objs.append(link)

            elif IURI.providedBy(field):
                link = BrokenLink()
                link.complete_information_with_external_path(obj,
                                                             fieldvalue)
                link_objs.append(link)

            elif IRichText.providedBy(field):
                content = fieldvalue.raw
                extract_and_append_link_objs(content, obj, link_objs)

    return link_objs


def extract_and_append_link_objs(content, obj, link_objs):
    links_and_relations_from_rich_text = extract_links_and_relations(
        content, obj)
    append_to_link_and_relation_information_for_different_link_types(
        links_and_relations_from_rich_text,
        link_objs, obj)


def iter_fields(portal_type):
    for schemata in iter_schemata_for_protal_type(portal_type):
        for name, field in getFieldsInOrder(schemata):
            if not getattr(field, 'readonly', False):
                yield (name, field, schemata)


def iter_schemata_for_protal_type(portal_type):
    if queryUtility(IDexterityFTI, name=portal_type):
        # is dexterity
        fti = getUtility(IDexterityFTI, name=portal_type)

        yield fti.lookupSchema()
        for schema in getAdditionalSchemata(portal_type=portal_type):
            yield schema


def extract_links_and_relations(content, obj):
    if not isinstance(content, basestring):
        return [[], [], []]
    # find links in page
    links_and_paths = extract_links_in_string(content, obj)
    # find and add broken relations to link_and_relation_information
    relation_uids = extract_relation_uids_in_string(content)

    links = links_and_paths[0]
    paths = links_and_paths[1]
    return [links, relation_uids, paths]


def extract_links_in_string(input_string, obj):
    # search for href and src
    regex = r"(href=['\"]?([^'\" >]+))|(src=['\"]?([^'\" >]+))"
    raw_results = re.findall(regex, input_string)

    output_urls = []
    output_paths = []
    for url in raw_results:
        # use actual link in findall tuple.

        url = url[1]

        if url.startswith('mailto:'):
            continue
        elif url.startswith('resolveuid/'):
            continue
        elif not urlparse(url).scheme:
            path = create_path_even_there_are_parent_pointers(obj, url)
            output_paths.append(path)
        else:
            output_urls.append(url)

    broken_paths = []
    # only append broken paths
    for path in output_paths:
        try:
            # unrestricted traverse needs utf-8 encoded string
            # but we want unicode in the end.
            api.portal.get().unrestrictedTraverse(path.encode('utf-8'))
        except Exception:
            broken_paths.append(path)

    return [output_urls, broken_paths]


def extract_relation_uids_in_string(input_string):
    regex = re.compile('resolveuid/(\w{32})', flags=re.IGNORECASE)

    return re.findall(regex, input_string)


def append_to_link_and_relation_information_for_different_link_types(
        links_and_relations_from_rich_text, link_and_relation_information,
        obj):
    links = links_and_relations_from_rich_text[0]
    uids = links_and_relations_from_rich_text[1]
    paths = links_and_relations_from_rich_text[2]

    for ext_link in links:
        link = BrokenLink()
        link.complete_information_with_external_path(obj, ext_link)
        link_and_relation_information.append(link)

    for uid in uids:
        link = BrokenLink()
        link.complete_information_with_internal_uid(obj, uid)
        link_and_relation_information.append(link)

    for path in paths:
        link = BrokenLink()
        link.complete_information_with_internal_path(obj, path)
        link_and_relation_information.append(link)


def create_excel_report(link_objs, base_uri):
    xlsx_file = create_excel_report_and_return_filepath(
        link_objs, base_uri)
    return xlsx_file


def create_excel_report_and_return_filepath(link_objs, base_uri):
    file_i = report_generating.ReportCreator()
    file_i.append_report_data(report_generating.LABELS,
                              base_uri,
                              BOLD &
                              CENTER &
                              DEFAULT_FONTNAME &
                              DEFAULT_FONTSIZE)
    file_i.append_report_data(link_objs, base_uri,
                              DEFAULT_FONTNAME &
                              DEFAULT_FONTSIZE)

    file_i.add_general_autofilter()
    file_i.cell_width_autofitter()
    file_i.safe_workbook()
    xlsx_file = file_i.get_workbook()
    return xlsx_file


def send_mail_with_excel_report_attached(email_address, plone_site_obj,
                                         total_time_fetching_external,
                                         xlsx_file, file_name):
    email_subject = 'Linkchecker Report'
    email_message = '''
    Dear Site Administrator, \n\n
    Please check out the linkchecker report attached to this mail.\n\n
    It took {}ms to fetch the external links for this report.\n\n
    Friendly regards,\n
    your 4teamwork linkcheck reporter\n\n\n
    '''.format(total_time_fetching_external)
    plone_site_path = '/'.join(plone_site_obj.getPhysicalPath())
    portal = api.content.get(plone_site_path)
    report_mailer_instance = report_mailer.MailSender(portal)
    report_mailer_instance.send_feedback(
        email_subject, email_message, email_address, xlsx_file, file_name)


def upload_report_to_filelistingblock(filelistingblock_url, xlsx_file, file_name):
    portal = api.portal.get()
    try:
        file_listing_block = portal.unrestrictedTraverse(path=filelistingblock_url.encode('utf-8'))
    except Exception as e:
        logger = setup_logger()
        logger.exception("Error while uploading report: upload location is not a valid path: {}".format(
            filelistingblock_url.encode('utf-8')))
    else:
        xlsx_file.seek(0)
        data = xlsx_file.read()

        file_ = plone.api.content.create(
            container=file_listing_block, type='File', title=file_name, file=data)
        file_.setFilename(file_name)
        transaction.commit()


def get_configs(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("--config",
                        help="Path to linkchecker config file.")
    parser.add_argument("--log",
                        help="Path to linkchecker log file.")
    args, unknown = parser.parse_known_args()
    path_to_config_file = args.config
    path_to_log_file = args.log

    if not os.path.isfile(path_to_config_file):
        exit()

    with open(path_to_config_file) as f_:
        config = json.load(f_)

    return config, path_to_log_file


def extract_config(config_file):
    portal_path = '/'.join(api.portal.get().getPhysicalPath())
    email_address = config_file[portal_path]['email']
    base_uri = config_file[portal_path]['base_uri']
    timeout_config = config_file[portal_path]['timeout_config']
    upload_location = config_file[portal_path].get('upload_location', '')

    return email_address, base_uri, timeout_config, upload_location


def get_file_name():
    return u'linkchecker_report_{}.xlsx'.format(
        time.strftime('%Y_%b_%d_%H%M%S', time.gmtime()))


def main(app, *args):
    configurations = get_configs(args)
    logger = setup_logger(configurations[1])
    logger.info('Linkchecker instance started as expected.')

    plone_site_objs = list(_get_plone_sites(app))
    config_file = configurations[0]

    logger.info(
        'Found site administrators email addresses for: %s' % ', '.join(
            [x['email'] for x in config_file.values()]))

    for plone_site_obj in plone_site_objs:
        setup_plone(app, plone_site_obj)
        email_address, base_uri, timeout_config, upload_location = extract_config(config_file)

        total_time_and_link_objs = get_total_fetching_time_and_broken_link_objs(
            int(timeout_config))

        time_for_fetching_external_links = total_time_and_link_objs[0]
        link_objs = total_time_and_link_objs[1]

        logger.info(
            'Finished going through all brains of "{}" and fetching for '
            'external Links. Total time fetching for external Links: '
            '{}ms.'.format(
                plone_site_obj.title,
                time_for_fetching_external_links))

        xlsx_file = create_excel_report(link_objs, base_uri)
        file_name = get_file_name()
        send_mail_with_excel_report_attached(
            email_address, plone_site_obj, time_for_fetching_external_links, xlsx_file, file_name)
        if upload_location:
            upload_report_to_filelistingblock(upload_location, xlsx_file, file_name)



class CellFormat(object):

    def __init__(self, format):
        self.format = format

    def __and__(self, cls):
        format = self.format
        format.update(cls.format)
        return CellFormat(format)

    def get_workbook_format(self, workbook):
        return workbook.add_format(self.format)


BOLD = CellFormat({'bold': True})
DATE_FORMAT = CellFormat({'num_format': 'mmmm d yyyy'})
ITALIC = CellFormat({'italic': True})
RED = CellFormat({'color': 'red'})
BLUE = CellFormat({'color': 'blue'})
CENTER = CellFormat({'align': 'center'})
DEFAULT_FONTNAME = CellFormat({'font_name': 'Courier New'})
DEFAULT_FONTSIZE = CellFormat({'font_size': 10})



import os

from setuptools import find_packages
from setuptools import setup

version = '1.0.1'

tests_require = [
    'ftw.builder',
    'ftw.testbrowser',
    'ftw.testing',
    'plone.app.testing',
    'plone.testing',
    # later versions of pandas require python-dateutil>=2.5.0
    'pandas < 0.23.0',
    # from numpy 1.17.0 only python >= 3.5 is supported
    'numpy < 1.17.0',
    'xlrd >= 0.9.0',
    'requests',
    'xlsxwriter',
    'ftw.simplelayout [contenttypes]',
]

extras_require = {
    'tests': tests_require,
}


setup(
    name='ftw.linkchecker',
    version=version,
    description='ftw.linkchecker',
    long_description=open('README.rst').read() + '\n' + open(
        os.path.join('docs', 'HISTORY.txt')).read(),

    # Get more strings from
    # http://www.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    keywords='ftw linkchecker',
    author='4teamwork AG',
    author_email='mailto:info@4teamwork.ch',
    url='https://github.com/4teamwork/ftw.linkchecker',
    license='GPL2',

    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['ftw'],
    include_package_data=True,
    zip_safe=False,

    install_requires=[
        'Plone',
        'setuptools',
        'ftw.upgrade',
        'xlsxwriter',
        'plone.api',
        'plone.app.relationfield',
        'plone.recipe.zope2instance',
    ],

    tests_require=tests_require,
    extras_require=extras_require,

    entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      [zopectl.command]
      check_links = ftw.linkchecker.command.checking_links:main
      """,
)



from zope.i18nmessageid import MessageFactory
import logging
import os
import sys

_ = MessageFactory('ftw.linkchecker')


def setup_logger(path=None):
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.INFO)

    if not path:
        return logger
    elif os.path.isfile(path):
        file_handler = logging.FileHandler(path)
        file_handler.setLevel(logging.INFO)
        logger.addHandler(file_handler)
        return logger
    else:
        logger.info('Invalid logging path!')
        sys.exit(1)


def initialize(context):
    pass



from Acquisition import aq_parent
from ftw.linkchecker.command import checking_links
from ftw.linkchecker.test_setup import add_archetype_link_to_plone_site
from ftw.linkchecker.test_setup import add_textarea_to_page
from ftw.linkchecker.test_setup import set_up_test_environment
from ftw.linkchecker.testing import ADDITIONAL_PAGES_TO_SETUP
from ftw.linkchecker.testing import LINKCHECKER_FUNCTIONAL
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from unittest2 import TestCase
import transaction


class FunctionalTestCase(TestCase):
    layer = LINKCHECKER_FUNCTIONAL

    def setUp(self):
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.textarea = add_textarea_to_page(self.portal)

    def grant(self, portal=None, *roles):
        if isinstance(portal, str):
            roles.append(portal)
            portal = self.portal
        elif isinstance(portal, list) or isinstance(portal, tuple):
            roles = list(portal)
            portal = self.portal
        else:
            portal = portal or self.portal

        setRoles(portal, TEST_USER_ID, list(roles))
        transaction.commit()


class ArchetypeFunctionalTestCase(TestCase):
    layer = LINKCHECKER_FUNCTIONAL

    def setUp(self):
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        plonefti = self.portal.portal_types.get('Plone Site')
        addable_types = list(plonefti.allowed_content_types) + ['Link']
        plonefti.allowed_content_types = tuple(addable_types)
        add_archetype_link_to_plone_site(self.portal)

        self.app = self.portal.aq_parent
        self.plone_site_objs = list(checking_links._get_plone_sites(self.app))

    def grant(self, portal=None, *roles):
        if isinstance(portal, str):
            roles.append(portal)
            portal = self.portal
        elif isinstance(portal, list) or isinstance(portal, tuple):
            roles = list(portal)
            portal = self.portal
        else:
            portal = portal or self.portal

        setRoles(portal, TEST_USER_ID, list(roles))
        transaction.commit()


class MultiPageTestCase(TestCase):
    layer = LINKCHECKER_FUNCTIONAL

    def setUp(self):
        self.request = self.layer['request']
        self.portal = self.layer['portal']

        # create list of all pages
        self.portals = [self.portal]
        for additional_page in ADDITIONAL_PAGES_TO_SETUP:
            self.portals.append(aq_parent(self.portal).get(
                additional_page['page_id']))

        for portal in self.portals:
            set_up_test_environment(portal)

        self.app = self.portal.aq_parent
        self.plone_site_objs = list(checking_links._get_plone_sites(self.app))

    def grant(self, portal=None, *roles):
        if isinstance(portal, str):
            roles.append(portal)
            portal = self.portal
        elif isinstance(portal, list) or isinstance(portal, tuple):
            roles = list(portal)
            portal = self.portal
        else:
            portal = portal or self.portal

        setRoles(portal, TEST_USER_ID, list(roles))
        transaction.commit()



config_file = {
    "/plone": {
        "email": "hugo.boss@4teamwork.ch",
        "base_uri": "http://example1.ch",
        "timeout_config": "1",
        "upload_location": "/plone/content-page/file-listing-block"
    },
    "/plone2": {
        "email": "berta.huber@gmail.com",
        "base_uri": "http://example2.ch",
        "timeout_config": "1"
    }
}
